//  GAMBIT: Global and Modular BSM Inference Tool
//  *********************************************
///  \file
///
///  Macros and related classes for declaring
///  scanner test functions.
///
///  *********************************************
///
///  Authors (add name and date if you modify):
///
///  \author Gregory Martinez
///          (gregory.david.martinez@gmail.com)
///  \date 2013 August
///        2014 Feb
///
///  \author Pat Scott
///          (p.scott@imperial.ac.uk)   
///  \date 2014 Dec
///
///  *********************************************

#ifndef INTERP_PLUGIN_HPP
#define INTERP_PLUGIN_HPP

#include "gambit/ScannerBit/scanner_utils.hpp"
#include "gambit/ScannerBit/factory_defs.hpp"
#include "gambit/ScannerBit/plugin_defs.hpp"
#include "gambit/ScannerBit/plugin_macros.hpp"

///\name Objective Plugin Macros
///Macros used by the objective plugins.
///@{
///Objective plugin declaration.  Is of the form:  objective_plugin(name, version)
#define interp_plugin(...)           INTERP_PLUGIN( __VA_ARGS__ )
///@}

#define __INTERP_SETUP__                                                                            \
using namespace Gambit::Scanner;                                                                    \
using Gambit::Printers::get_point_id;                                                               \
                                                                                                    \
inline std::vector<std::string> &get_keys() {return get_input_value<std::vector<std::string>>(0);}  \
inline unsigned int &get_dimension() {return get_input_value<unsigned int>(1);}                     \
                                                                                                    \
void *get_purpose(const std::string &purpose)                                                       \
{                                                                                                   \
    void *ptr = (get_input_value<Factory_Base>(2))(purpose);                                        \
    static_cast <Function_Base<void(void)>*>(ptr)->setPurpose(purpose);                             \
    static_cast <Function_Base<void(void)>*>(ptr)->setPrinter(get_printer().get_stream());          \
    static_cast <Function_Base<void(void)>*>(ptr)->setPrior(&get_prior());                          \
    assign_aux_numbers(purpose, "pointID", "MPIrank");                                              \
                                                                                                    \
    return ptr;                                                                                     \
}                                                                                                   \

#define INTERP_PLUGIN(plug_name, ...)                                                               \
    GAMBIT_PLUGIN_INITIALIZE(__INTERP_SETUP__, plug_name, interp, __VA_ARGS__)                      \

#endif
